/**
 * $Revision: 1.15 $
 * $Date: 2006/03/30 11:34:27 $
 */

#ifndef __INDUCERH__
#define __INDUCERH__

#include "../MLPCommon/Logger.h"
#include "../MLPDataset/IDataSet.h"
#include "Classifier.h"
#include <fstream>
#include <string>
#include <sstream>
#include <vector>

using namespace std;

/**
 * Abstrakcyjna klasa, ktorej implementacje sluza do tworzenia klasyfikatorow.
 * @author Marek Grzes'
 */
class IInducer{
friend class IDTInducerAP;

public:
	IInducer();

	virtual ~IInducer();
	
	/**
	 * Dla podanego zbioru trenujacego zwraca klasyfikator
	 * utworzony na podstawie tego zbioru. Tej metody nie trzeba
	 * implementowac w klasach potomnych. Nalezy implementowac metode
	 * ActualTrain.
	 * @see ActualTrain
	 */
	virtual IClassifier *Train(const IDataSet *trainingDataSet);

	/**
	 * Pozwala na ustawianie okreslonych parametrow inducera.
	 * @param sParamName - nazwa parametru
	 * @param sParamValue - wartosc parametru
	 * @return TRUE - parametr ustawiony, FALSE - blad, np. parametr lub jego wartosc nie sa obslugiwane
	 */
	virtual bool SetParam(string sParamName, string sParamValue);
	
	/**
	 * Metoda ustawia pole m_sResultsFileStem
	 * @see m_sResultsFileStem
	 */
	void SetResultsFileStem(string rFileStem){
		m_sResultsFileStem = rFileStem;
	}
	
	/**
	 * Pozwala na ustawianie parametrow MLPSubParam. Implementuja ja tylko te inducery,
	 * ktore maja takie parametry.
	 * @param sParentParamName - nazwa parametru MLPParam, dla ktorego ustawiany
	 * jest dany MLPSubParam.
	 * @param sParamName - nazwa parametru MLPSubParam
	 * @param sParamValue - wartosc parametru MLPSubParam
	 * @return TRUE - parametr ustawiony, FALSE - blad, np. parametr lub jego wartosc nie sa obslugiwane
	 */
	virtual bool SetSubParam(string sParentParamName, string sSubParamName, string sSubParamValue){
		return false;
	};

	/**
	 * Metoda zwraca wszystkie parametry inducera. Na pozycjach parzystych
	 * zwracanego wektora sa nazwy parametrow a na pozycjach nieparzystych odpowiadajace
	 * im wartosci.
	 */
	virtual vector<string> GetParams() const;
	
	/**
	 * Zwraca wszystkie paramtery dodatkowe MLPSubParam, ktore odpowiadaja
	 * przekazanemu w paremetrze sParamName parametrowi MLPParam i jego
	 * wartosci sParamValue. Metode ta implementuja tylko te inducery, ktore
	 * maja parametry MLPSubParam.
	 */
	virtual vector<string> GetSubOptions(string sParamName, string sParamValue) const{
		vector<string> res;
		return res;
	};
	
	/**
	 * @see m_bCostOfClassification
	 */
	virtual bool IsCostOfClassification()const{
		return m_bCostOfClassification;
	}
	
	/**
	 * @see m_bCostOfAttrs
	 */
	virtual bool IsCostOfAttrs()const{
		return m_bCostOfAttrs;
	}
	
	/**
	 * Drukuje przykladowe tagi XML do wstawienia do pliku konfiguracyjnego dla danego inducera.
	 * @param ostr - strumien wyjsciowy
	 * @param nsp - liczba spacji poprzedzajacych drukowany TAG XML
	 */
	virtual void PrintParamsXML(ostream &ostr, int nsp);

	virtual string GetName()=0;
	
	void SetLogLevel(int level){
		m_iLogLevel = level;
	}
		
protected:
	/**
	 * Zbior danych uczacych. Wartosc tego wskaznika obowiazuje w czasie dzialania
	 * metody Train. Powinien byc ustawniony na poczatku metody Train i obowiazuje
	 * do zakonczenia jej dzialania. Tego wskaznika nie wolno usuwac.
	 */
	const IDataSet* m_pDataset;

	/**
	 * Strumien wyjsciowy, do ktorego drukowane sa informacje diagnostyczne
	 * danego inducera.
	 */
	ofstream	m_ofsLogFile;

	/**
	 * Paramer przechowujacy poczatek podstawy nazwy plikow wynikowych dla danego
	 * eksperymentu. Nazwa ta jest potrzebna to generowania nazw plikow dla
	 * dodaktowych informacji zapisywanych do plikow.
	 * <b>Parametr jest podawany tak, ze dla eksperymentow z kroswalidacja
	 * zawiera on juz suffix _FOLDn</b>
	 */
	string	m_sResultsFileStem;
	
	/**
	 * 0 - poziom zerowy, brak informacji.
	 * 1 - pierwszy poziom 
	 * 2 - zapis szczegolowych informacji diagnostycznych
	 */
	int m_iLogLevel;

	/**
	 * Nauczanie ma uwzgledniac koszt atrybutow - kiedy wartos true.
	 */
	bool m_bCostOfAttrs;
	
	/**
	 * Nauczanie ma uwzgledniac koszt blednej klasyfikacji - kiedy wartos true.
	 */
	bool m_bCostOfClassification;

	/**
	 * Otwiera strumien m_ofsLogFile. Jesli byl on juz otwarty to jest najpierw zamykany.
	 * @param fname - sciezka do pliku do otwarcia
	 */
	void OpenLogFile(string fname);
	void OpenLogFile();
	
	/**
	 * Metoda wywlolywana po wyjsciu z metody ActualTrain() w celu np. zwolnienia 
	 * zasobow.
	 */
	virtual void PostTrain(const IDataSet *trainingDataSet){
	}
	
	/**
	 * Metoda wywlolywana przed wejsciem do metody ActualTrain()
	 */
	virtual void PreTrain(const IDataSet *trainingDataSet);

private:
	/**
	 * Dla podanego zbioru trenujacego zwraca klasyfikator
	 * utworzony na podstawie tego zbioru.
	 */
	virtual IClassifier *ActualTrain(const IDataSet *trainingDataSet)=0;
	
};

#endif


