/**
 * $Revision: 1.21 $
 * $Date: 2006/04/19 23:53:46 $
 */

#ifndef __IEVOLUTIONARYALGH__
#define __IEVOLUTIONARYALGH__

#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <deque>
#include "IIndividual.h"

#ifdef MLPEAOPERATORSTEST
#include <map>
#endif

using namespace std;

/**
 * @author Marek Grzes'
 */
class IEvolutionaryAlg{
typedef void (IEvolutionaryAlg::*TSELECTION)(deque<IIndividual*> &p);

public:
	virtual vector<string> GetParams() const;
	virtual bool SetParam(string sParamName, string sParamValue);
	virtual void PrintParamsXML(ostream &ostr, int nsp);
	virtual void PrintSubParamsXML(ostream &ostr, int nsp);

	IEvolutionaryAlg(void);
	virtual ~IEvolutionaryAlg(void);
	
protected:

	/**
	 * Zmienna wskazujaca koniecznosc zapisania do pliku informacji
	 * na temat zmian wartosci dopasowania i parametrow, od ktorych ono zalezy.
	 */
	bool	m_bFitnessLog;

	/**
	 * Populacja osobnikw.
	 */
	deque<IIndividual*> m_dqPopulation;

	/**
	 * Najlepszy znaleziony osobnik.
	 */
	IIndividual* m_BestIndividual;

	unsigned	m_uMaxIterations;
	unsigned	m_uMaxIdleIterations;
	unsigned	m_uMinIterations;
	unsigned	m_uSizeOfPop;
	
	/**
	 * Prawodpodobienstwo mutacji w procentach, zakres wartosci [0,100].
	 */
	int	m_iProbabilityOfMutation;
	
	/**
	 * Prawodpodobienstwo krzyzowania w procentach, zakres wartosci [0,100].
	 */
	int	m_iProbabilityOfCrossOver;
	
	/** 
	 * Rodzaj selekcji:
	 * 0 - roulette wheel selection
	 * 1 - ranking linear selection
	 */
	int	m_iSelection;

	/**
	 * Napor selekcyjny przy liniowej selekcji porzadkowej.
	 * Zakres wartosci: [1;2]
	 * 1 - brak naporu selekcyjnego
	 * 2 - maksymalny napor selekcyjny
	 */
	double	m_dSelectivePressure;
	
	/**
	 * Operator krzyzowania populacji m_dqPopulation
	 * @see m_dqPopulation
	 */
	virtual void CrossoverOperator()=0;

	/**
	 * Metoda jest wywolywana na koniec funkcji Run, tuz przed zwroceniem
	 * najlepszego osobnika. Mozna w niej zaimplementowac jakies
	 * mechanizmy poprawiania najlepszego osobnika.
	 */
	virtual void ImproveTheBestIndividual(IIndividual *pBest){
	}

	/**
	 * Operator mutacji na osobnikach populacji m_dqPopulation. 
	 * @see m_dqPopulation
	 */
	virtual void MutationOperator(int iterNum)=0;
	
	/**
	 * Wybranie osobnikow do reprodukcji. Wybor odbywa sie zgodnie z zasada 
	 * selekcji ruletkowej (ang. roulette wheel selection). Do nowej populacji
	 * wybieranych jest n-1 jeden osobnikow, gdzie n jest liczebnoscia populacji.
	 * n-tym elementem jest najlepszy znaleziony do tej pory osobnik. 
	 * @param p - populacja
	 */
	virtual void RouletteWheelSelection(deque<IIndividual*> &p);

	/**
	 * Wybranie osobnikow do reprodukcji. Wybor odbywa sie zgodnie z zasada 
	 * porzadkowej selekcji liniowej (ang. ranking linear selection).Do nowej
	 * populacji wybieranych jest n-1 jeden osobnikow, gdzie n jest liczebnoscia
	 * populacji.
	 * n-tym elementem jest najlepszy znaleziony do tej pory osobnik. 
	 * @param p - populacja
	 */
	virtual void RankingLinearSelection(deque<IIndividual*> &p);
		
	/**
	 * Wybranie osobnikow do reprodukcji. Wybor odbywa sie zgodnie z zasada 
	 * selekcji okreslonej parametrem m_iSelection. Znaleziony do tej pory
	 * najlepszy osobnik jest wstawiany na pierwszej pozycji w nowej populacji.
	 * @see m_dqPopulation
	 */
	virtual void SelectIndividualsToReproduce();
	
	/**
	 * Dla podanego w parametrze osobnika oblicza dopasowanie.
	 * @param individual - osobnik, dla ktorego dopasowanie ma byc wyznaczone.
	 */
	virtual void EvaluateIndividualsFitness(IIndividual* individual)=0;

	/**
	 * Dla podanej w parametrze populacji obliczane jest dopasowanie.
	 * @param population - populacja, dla ktorej osobnikow ma byc wyznaczone 
	 * dopasowanie
	 */
	virtual void EvaluateFitness(deque<IIndividual*> &population);

	/**
	 * Metoda do utworzenia populacji poczatkowej.
	 * @see ActualCreateInitialPopulation
	 */
	virtual void CreateInitialPopulation();
	
	/**
	 * Metoda wywolywana po utworzeniu populacji poczatkowej.
	 */
	virtual void PostCreateInitialPopulation()=0;
	
	/**
	 * Metoda wywolywana przed rozpoczeciem generowania populacji poczatkowej.
	 */
	virtual void PreCreateInitialPopulation()=0;
	

	/**
	 * Wypisanie informacji o wartosci dopasowania i parametrow, od ktorych ono zalezy
	 * w jednej populacji. Funkcja ta jest wywolywana tylko wtedy kiedy m_bFitnessLog ma 
	 * wartosc true. W implementacjach tej metody jest rozsztrzygane co ma byc drukowane
	 * przez ta funkcje (np. koszt).
	 * @param p - populacja
	 * @param iternum - numer iteracji
	 * @param ostr - referencja do strumienia
	 * @see m_bFitnessLog
	 */
	virtual void PrintFitnessLog(deque<IIndividual*> &p,IIndividual* bestind, unsigned iternum, ostream &ostr)=0;
	
	/**
	 * Informacje o jednym osobniku. Do wykorzystania w logach. To gdzie
	 * bedzie to wypisane zalezy do klas potomnych, powinno byc to do wyisane do pliku, do
	 * ktorego drukowane sa logi.
	 * @param prefix - tekst do wypisania przed parametrami osobnika
	 * @param ind - osobnik
	 * @param bPrintStructure - wskazuje czy ma byc wydrukowane takze struktura osobnika (np.
	 * struktura drzewa decyzyjnego albo sieci neuronalnej)
	 */
	virtual void PrintIndividualToLog(string prefix, IIndividual* ind, bool bPrintStructure=false)=0;
	
	/**
	 * Metoda stanowiaca algorytm ewolucyjny.
	 * @param sResultsFileStem - podstawa nazwy pliku wynikow dla eksperymetnu biezacego 
	 * algorytmu na biezacym zbiorze danych, biezacym uruchomieniu i podziale kroswalidacji
	 * (jednym slowem jest to nazwa unikalna dla danego uruchomienia).
	 * @param iDebugLevel - poziom logowania
	 * @param  ofLogStream - przekazany z klasy nadrzednej strumien, do ktorego drukowane sa logi
	 * @return zwraca najlepszy znaleziony osobnik
	 */
	virtual IIndividual* Run(const string sResultsFileStem, int iDebugLevel, ofstream &ofLogStream);

	/**
	 * Wywolywana na nowej populacji, po operatorach ewolucyjnych.
	 * <p><b>UWAGA!</b> Po wywolaniu tej funkcji nie jest juz uaktualniana
	 * wartosc dopasowania w osobnikach, trzeba ja uaktualnic w tej
	 * funkcji jesli zachodzi taka potrzeba.</p>
	 */
	virtual void PostEAOperations(deque<IIndividual*> &p)=0;

	/**
	 * Drukuje domyslne wartosci wszystkich parametrow jakie mozna okreslic da obiektow tej
	 * klasy.
	 * @param ostr - strumien wynikowy
	 * @param sParamName - nazwa tag'a XML, pod ktorym beda wydrukowane wszystkie parametry
	 */
	virtual void PrintXML(ostream &ostr, int nsp, string sParamName);
	
	/**
	 * Wskaznik do funkcji dokonujacej seleckji
	 */
	TSELECTION f_ptSelection;
	
	/**
	 * Funkcja towrzy projekt Gnuplota dla wykresu statystyk dopasowania. Funkcja wykonuje
	 * sie zaleznie od pola m_bFitnessLog. Metoda ta musi byc implementowana przez klasy
	 * potomne, bo projekty gnuplota zaleza od tego co wydrukuje fukcja PrintFitnessLog
	 * do pliku z danymi.
	 * @param datafilename - nazawa pliku z danymi do wykresu
	 * @param outfilename - nazwa pliu wyjsciowego .plt
	 * @see m_bFitnessLog
	 * @see PrintFitnessLog
	 * @see PrintFitnessLogHeader
	 */
	virtual void CreateGnuplotProjects(string datafilename, string outfilenamestem)=0;

	/**
	 * Drukuje naglowek dla pliku za danymi dla wykresow gnuplota, obrazujacych
	 * zmiany funkcji fitness i jej parametrow w kolejnych iterachach
	 */
	virtual void PrintFitnessLogHeader(ofstream &ofFlog)=0;

// ------------------------------- OP TEST BEGIN ----------------------------
#ifdef MLPEAOPERATORSTEST
	/**
	 * Maksymalny fitness z poprzedniej iteracji.
	 */
	double m_dMaxFitness;
	ofstream m_ofsOperatorTest;
	ofstream m_ofsOperatorTestBestIndv;
	
	/**
	 * Strumien do zapisywania informacji o tym jak czesto sie wykonana mutacja
	 * oraz jak czesto dawala poprawe.
	 */
	ofstream m_ofsMutationCounting;

	/**
	 * Do zliczania efektywnosci operatorow w ramach jednej iteracji.
	 * Pierwszy element pary to ilosc popraw, a drugi zlicza ile razy wywolany byl
	 * kazdy z operatorow.
	 */
	map<string,pair<int,int> > m_mOpEfficiency;
	map<string,pair<int,int> > m_mOpEfficiencyBestIndv;
	
	/**
	 * Wektor z nazwami operatorow ewolucyjnych. Kolejnosc w tym wektorze ma znaczenie.
	 * Ta sama kolejnosc operatorow uwzglednia funkcja drukujaca projekt gnuplota
	 * do wygenerowania wykresow.
	 * Wektor ten powinien byc inicjowany np. w konstruktorze konkretnej implementacji alg. ewolucyjnego
	 * np. w klasie CEAIDTInducerAP lub CEAIDTInducerOB. 
	 * @see OPTEST_CreateGnuplotProjects
	 * @see CEAIDTInducerAP
	 * @see CEAIDTInducerOB
	 */
	vector<string>	m_vOpNames;
	
	virtual void OPTEST_OperatorsTestLog(IIndividual &indv, string sOpName)=0;
	virtual void OPTEST_PrintOperatorsTestLog(unsigned iterNum);
	virtual void OPTEST_PrintOperatorsNames();
	virtual void OPTEST_OpenFiles(string sFilestem);
	
	/**
	 * Dwa ostatnie parametry tej metody powinny dostawac nazwe pliku plt bez rozszerzenia.
	 */
	void OPTEST_CreateGnuplotProjects(string sOpTestsFileData, string sBestFileData, string sMutationFileData, string sOpTestsPltFileStem, string sBestPltFileStem, string sMutationPltFileStem);
	
	void OPTEST_WholePopulationGnuplotProject(string sOpTestsFileData, string sOpTestsPltFileStem, int opIndex,int iFirstIndex);
	void OPTEST_TheBestGnuplotProject(string sBestFileData, string sBestPltFileStem, int opIndex,int iFirstIndex);
	void OPTEST_MutationGnuplotProject(string sMutationFileData, string sMutationPltFileStem);
#endif
// ------------------------------- OP TEST END ----------------------------

private:
	vector<double> m_vdDist;
	vector<size_t> m_vtMulti;
	
	/**
	 * Wybranie osobnikow do reprodukcji na podstawie dystrybuanty m_vdDist. Po 
	 * wykonaniu tej metody w zmiennej p jest juz nowa populacja.
	 * @see m_vdDist
	 */
	void SelectIndividuals(deque<IIndividual*> &p);
	
	/**
	 * Metoda do utworzenia populacji poczatkowej. Ta metoda powinna byc
	 * implementowana w klasach potomnych. Metody CreateInitialPopulation
	 * nie trzeba implementowac, bo jest to metoda opakowujaca.
	 * <b>Metoda ta jest prywatana, zeby nikomu nie przyszlo do glowy ja wywolywac.</b>
	 * @see CreateInitialPopulation
	 */
	virtual void ActualCreateInitialPopulation()=0;

};


#endif

