#ifndef __CDTTREENODEH__
#define __CDTTREENODEH__

#include "../MLPDataset/AttrValue.h"
#include "ITest.h"
#include <vector>
#include <deque>

/**
 * Klasa wezla drzewa decyzyjnego.
 * @author Marek Grzes
 */
class CDTreeNode{
public:
	
	/**
	 * Koszt klasyfikcaji, dla poddrzewa zaczynajacego sie w biezacym wezle.
	 * W korzeniu jest to koszt klasyfikacji calego drzewa.
	 * <b>UWAGA!</b> Jest to suma kosztu a nie srednia, bo srednia
	 * trzeba liczyc tylko w korzeniu danego poddrzewa.
	 * @see m_uNExamples
	 * @see GetCostOfClassificationOnTrainDS
	 */
	float m_fCostOfClassification;

	/**
	 * W korzeniu kazdego poddrzewa jest koszt atrybutow uzytych przy
	 * reklasyfikacji tych przykladow zbioru uczacego, ktore laduja w tym poddrzewie.
	 * Dla danego przykladu kosztem atrybutow jest suma kosztu wszystkich cech 
	 * wystepujacych na sciezce od korzenia drzewa do liscia, ktory ten przyklad
	 * klasyfikuje. Wartosc kazdej cech jest brana dokladnie do tej sumy <b>(czyli
	 * jak jakas cecha wystepuje wiecej niz raz, to jest liczona dokladnie raz)</b>.
	 * 
	 * <b>Wartosc m_fCostOfAttrs w korzeniu drzewa jest kosztem aktybutow przy 
	 * reklasyfikacji calego zbioru uczacego.</b> Zeby otrzymac srednia trzeba
	 * podzielic przez m_uNExamples tak jak w m_fCostOfClassification.
	 * @see m_fCostOfClassification
	 * @see m_uNExamples
	 */
	float m_fCostOfAttrs;

	/**
	 * Jest to liczba, ktora wskazuje ile przykladow spaca do poddrzewa
	 * zaczynajacego sie w biezacym wezle. Liczna ta jest wykorzystywana
	 * przy liczeniu sredniego kosztu blednej klasyfikacji.
	 *
	 * costOfClassification = m_fCostOfClassification / m_uNExamples
	 */
	unsigned m_uNExamples;

	/**
	 * Zwraca koszt atrybutow dla danego poddrzewa (liczony na zbiorze uczacym).
	 * @see m_fCostOfAttrs
	 * @see m_uNExamples
	 */
	virtual float GetCostOfAttributesOnTrainDS() const {
		return m_fCostOfAttrs / (float) m_uNExamples;
	}

	/**
	 * Zwraca koszt klasyfikacji dla danego poddrzewa (liczony na zbiorze uczacym).
	 * @see m_fCostOfClassification
	 * @see m_uNExamples
	 */
	virtual float GetCostOfClassificationOnTrainDS() const {
		return m_fCostOfClassification / (float) m_uNExamples;
	}
	
	/**
	 * Zmienna wskazujaca jaki jest rozmiar poddrzewa poczynajac od
	 * biezacego wezla. Wartosc tej zmiennej ustawiana jest w metodzie
	 * EstimateErrors. Dlatego z tego pola mozna korzystac dopiero
	 * po wywolaniu metody EstimateErrors. Liczba ta uwzglednia takze,
	 * liscie drzewa, dlatego w lisciu bedzie to wartosc rowna 1.
	 * W korzeniu wartosc ta reprezentuje rozmiar drzewa.
	 * @see EstimateErrors
	 */
	int m_iSubtreeSize;

	/**
	 * Liczba przykladow pokrywanych przez wezel lub lisc.
	 */ 
	double m_dN;

	/**
	 * Liczba niepoprawnie pokrywanych przykladow.
	 */
	double m_dE;

	/**
	 * Blad wyestymowany.
	 */
	double m_dEstE;

	bool m_bLeaf;

	/**
	 * Most frequent class at this node.
	 */
	IAttrValue* m_pIAClass;

	ITest* m_pITest;
	vector<CDTreeNode*> m_vBranch;

	/**
	 * Czesc zbioru testowego pokrywana przez dany wezel. 
	 */
	deque<CDataObject*> m_deqCI;

	virtual void ClearDeqCI();

	/**
	 * Zwraca bd na zbiorze testowym.
	 * @return wynik w procentach [0;100]
	 */
	double GetErrorOnTestDS();

	/**
	 * Zwraca bd pesymistyczny (estymowany).
	 * @return wynik w procentach [0;100]
	 */
	double GetEstimateError();

	/**
	 * Zwraca liczbe wezlow w drzewie, ktorego korzeniem jest wezel biezacy.
	 */
	virtual unsigned GetNodeCount();

	/**
	 * Zwraca liczbe lisci drzewa, ktorego korzeniem jest wezel biezacy.
	 */
	virtual unsigned GetLeafsCount()const;
	
	/**
	 * Zwraca glebokosc drzewa, ktorego korzeniem jest wezel biezacy.
	 */
	virtual unsigned GetDepth();

	/**
	 * Zwraca wezel drzewa (lisc lub wezel wewnetrzny) wybrany
	 * z rozkladem rownomiernym.
	 */
	virtual CDTreeNode* GetNodeOrLeafEvenProb();
	virtual void GetNodeOrLeafEvenProb(vector<CDTreeNode*> &vCDT);
	
	/**
	 * Zwraca lisc drzewa wybrany z rozkladem rownomiernym.
	 */
	virtual CDTreeNode* GetLeafEvenProb();
	virtual void GetLeafEvenProb(vector<CDTreeNode*> &vCDT);

	/**
	 * Zwraca wezel wewnetrzny drzewa wybrany z rozkladem rownomiernym.
	 */
	virtual CDTreeNode* GetNodeEvenProb();
	virtual void GetNodeEvenProb(vector<CDTreeNode*> &vCDT);

	/**
	 * Zwraca wezel wewnetrzny drzewa wybrany z rozkladem rownomiernym. Przeszukiwanie odbywa sie 
	 * w dziedzinie wezlow majacych t potomkow.
	 */
	virtual CDTreeNode* GetNodeEvenProb(unsigned uSize);
	virtual void GetNodeEvenProb(vector<CDTreeNode*> &vCDT, unsigned uSize);

	/**
	 * Znajduje losowy lisc w drzewie. Zwraca wskaznik.
	 * Funkcja zwraca tylko wezel. Nie ma informacji jakie testy
	 * sa na sciezce do niego prowadzacej.
	 * @return wskaznik na lisc - nie moze byc zwrocona wartosc NULL
	 */
	virtual CDTreeNode* FindRandomLeaf();

	/**
	 * Znajduje losowy wezel w drzewie. Zwraca wskaznik.
	 * Brak informacji o testach jakie sa na sciezce do niego
	 * prowadzacej oraz w jego potomkach.
	 * @return wskasnik na wezel lub NULL kiedy drzewo sklada sie z jednego
	 * wezla, ktory jest lisciem.
	 */
	virtual CDTreeNode* FindRandomNode();
	
	/**
	 * Znajduje losowy wezel sposrod wezlow o n potomkach.
	 * @return wskaznik na wezel o n potomkach lub NULL kiedy na przeszukiwanej
	 * galezi nie znaleziono zadnego wezla o n potomkach.
	 */
	virtual CDTreeNode* FindRandomNode(unsigned t);
	
	/**
	 * Znajduje losowy wezel (wezel wewnetrzny lub lisc) w drzewie.
	 * @return wskanik na wezel - nie moze byc tu zwrocona wartosc NULL
	 */
	virtual CDTreeNode* FindRandomNodeOrLeaf();

	/**
	 * Znajduje w drzewie wezel, w ktorym jest przecinany podany w parametrze dipol.
	 * @return wskaznik na wezel lub NULL kiedy nie ma w drzewie testu przecinajacego
	 * zadany dipol.
	 */
	virtual CDTreeNode* FindNodeCuttingDipol(pair<CDataObject*,CDataObject*> p);
	
protected:
	virtual CDTreeNode* FindRandomNode(vector<CDTreeNode*> &vCDT);
	virtual CDTreeNode* FindRandomNode(unsigned t, vector<CDTreeNode*> &vCDT);
	virtual CDTreeNode* FindRandomNodeOrLeaf(vector<CDTreeNode*> &vCDT);
public:
	CDTreeNode();
	CDTreeNode(const CDTreeNode& templ);
	virtual ~CDTreeNode();

};

#endif

